import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl

font_size = 15
params = {
    'image.origin': 'lower',
    'image.interpolation': 'nearest',
    'image.cmap': 'gray',
    'axes.grid': False,
    'savefig.dpi': 500,
    'axes.labelsize': font_size,
    'axes.titlesize': font_size,
    'lines.markersize': 5,
    'lines.linewidth': 1,
    'font.size': font_size,
    'legend.fontsize': font_size*0.85,
    'xtick.labelsize': font_size*0.85,
#    'text.usetex': True,
    'ytick.labelsize': font_size*0.85,
    'font.family': 'serif',
}

mpl.rcParams.update(params)
# mpl.rc('font', **{'family':'serif', 'serif':['Computer Modern Roman'],
#     'monospace': ['Computer Modern Typewriter']})
fig_save = 'png'

cases = ['saved/0_Reference','NewTest']
cases_names = ['Reference', 'New test']

colors = ['black', 'red']
line_style = ['-','--',':','-.']
y_axis_limits = [[1, 2],[-2, 2]]
H = 0.0127

# ============================ Velocity profiles ============================ #
x_lab = ['$x/H+2U/{{U}_{\mathrm{ref}}}$','$x/H+20V/{{U}_{\mathrm{ref}}}$']
fig_names = ['U','V']
locs = [-4, 1, 4, 6, 10]
minus_plus = ['m', '', '', '', '']
factor = [2, 20]

criteria = 5
error_rel_max = np.zeros((len(x_lab),len(locs)))

for i in range(len(x_lab)):
    
    fig, ax = plt.subplots(figsize=(10, 5))
    ax.set_aspect('equal', adjustable='box')
    # plot the backward facing step geometry
    ax.plot([-130,0,0,50], [1,1,0,0], '-', color = 'black', linewidth = 3)
    ax.plot([-130,50], [9,9], '-', color = 'black', linewidth = 3)
    ax.fill_between([-130, 0], -1, 1, facecolor='gray')

    for j in range(len(locs)):
        
        for k in range(len(cases)):
            
            address = cases[k] + '/postProcessing/sample/2000/'
            file_name_num = address + 'x_by_h_' + minus_plus[j] + \
                        str(abs(locs[j])).zfill(2) + '_U.xy'
            num_data = np.loadtxt(file_name_num)

            u_ref = np.loadtxt(address + 'Uref_U.xy')
            u_ref = u_ref[1]

            ax.plot([locs[j],locs[j]], [int(locs[j]<0),9], ':', color = 'black',
                    linewidth = 0.5)
            ax.plot(locs[j] + factor[i]*num_data[:,i+1]/u_ref, num_data[:,0]/H,
                    line_style[k], color = colors[k], label = cases_names[k])

            if k == 0:
                num_data_Ref = num_data[:,i+1]
            else:
                error_rel_max[i,j] = np.max(abs(num_data_Ref - num_data[:,i+1])/u_ref*100)
                # error_rel_max[i,j] = np.max(abs(np.divide(num_data_Ref - num_data[:,i+1], 
                #                                           num_data_Ref+1e-4)))*100

        file_name_exp = 'Experiments/profiles_x_by_h_' + minus_plus[j] + \
                        str(abs(locs[j])).zfill(2) + '.dat'
        exp_data = np.loadtxt(file_name_exp,comments='#')
        ax.plot(locs[j] + factor[i]*exp_data[:,i+2], exp_data[:,1],
                marker='o', markerfacecolor='None', markeredgecolor='blue',
                markersize=5, linestyle = 'None', 
                label = 'Experiment')

    handles, labels = ax.get_legend_handles_labels()
    ax.set_xlim((-6, 12))
    ax.set_ylim((-0.02, 9.04))
    ax.set_xlabel(x_lab[i])
    ax.set_ylabel('$y/H$')
    display = (0,1,11)
    ax.legend([handle for i,handle in enumerate(handles) if i in display],
              [label for i,label in enumerate(labels) if i in display],
               loc='best')
    plt.savefig(fig_names[i] + '.' + fig_save, format=fig_save)
    # plt.show()

error_max = np.max(error_rel_max)
print('\nThe passing criteria is ' + str(criteria) + '%.')
print('Maximum relative error of velocity profiles is ' + 
      "{:.3f}".format(error_max) + '% and therefore the test',
      'Passes.' if error_max < criteria 
      else 'Fails. Please have a look at the results.')
# =========================================================================== #




# ================================= Cp & Cf ================================= #
rho = 1
num_name = ['p', 'wallShearStress']
exp_name = ['cp', 'cf']
wall_name = ['lower', 'upper']
plus_minus = [1, -1]
y_lab = ['$C_p$','$C_f$']

for i in range(len(num_name)):
    error_rel_max = np.zeros(len(wall_name))

    for j in range(len(wall_name)):
    
        fig, ax = plt.subplots(figsize=(6, 5))
        # ax.set_aspect('equal', adjustable='box')
        # plot the backward facing step geometry
        
        for k in range(len(cases)):
            
            address = cases[k] + '/postProcessing/surface/2000/'
            file_name_num = address + num_name[i] + '_' + wall_name[j] + \
                        'Wall.raw'
            num_data = np.loadtxt(file_name_num)

            u_ref = np.loadtxt(address.replace('surface','sample') + 'Uref_U.xy')
            u_ref = u_ref[1]

            nondim_data = plus_minus[i]*num_data[:,3]/(0.5*rho*u_ref**2)
            ax.plot(num_data[:,0]/H, nondim_data, line_style[k], 
                    color = colors[k], label = cases_names[k])

            if k == 0:
                num_data_Ref = nondim_data
            else:
                error_rel_max[j] = np.max(abs((num_data_Ref - nondim_data)/np.mean(num_data_Ref)))*100
                #error_rel_max[j] = np.max(abs(np.divide(num_data_Ref - nondim_data, num_data_Ref)))*100

        if i + j < 2: # no experimental data for cf on upper wall
            file_name_exp = 'Experiments/' + exp_name[i] + '_' + \
                            wall_name[j] + '.dat'
            exp_data = np.loadtxt(file_name_exp,comments='#')
            ax.plot(exp_data[:,0], exp_data[:,1], marker='o',
                    markerfacecolor='None', markeredgecolor='blue',
                    markersize=5, linestyle = 'None', label = 'Experiment')

        ax.set_xlim((-5, 30))
        # ax.set_ylim((-0.02, 9.04))
        ax.set_xlabel('$x/H$')
        ax.set_ylabel(y_lab[i])
        ax.legend(loc='best')
        plt.savefig(exp_name[i] + '_' + wall_name[j] + '.' + fig_save, 
                    format=fig_save, bbox_inches='tight')

    error_max = np.max(error_rel_max)
    print('Maximum relative error of ' + exp_name[i] + ' is '
    "{:.3f}".format(error_max) + '% and therefore the test', 
    'Passes.' if error_max < criteria 
    else 'Fails. Please have a look at the results.')
# =========================================================================== #
