import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import os

criteria = 1 #Percent
fig_save = 'png'

font_size = 15
params = {
    'image.origin': 'lower',
    'image.interpolation': 'nearest',
    'image.cmap': 'gray',
    'axes.grid': False,
    'savefig.dpi': 500,
    'axes.labelsize': font_size,
    'axes.titlesize': font_size,
    'lines.markersize': 5,
    'lines.linewidth': 1,
    'font.size': font_size,
    'legend.fontsize': font_size*0.85,
    'xtick.labelsize': font_size*0.85,
#    'text.usetex': True,
    'ytick.labelsize': font_size*0.85,
    'font.family': 'serif',
}

mpl.rcParams.update(params)
# mpl.rc('font', **{'family':'serif', 'serif':['Computer Modern Roman'],
#     'monospace': ['Computer Modern Typewriter']})

cases = [ f.name for f in os.scandir('./saved') if f.is_dir() ]
cases.sort()
cases_names = cases

linestyle_tuple = [
     #('solid',                 (0, ()))]
     ('solid',                 (0, ())),

     ('loosely dotted',        (0, (1, 5))),
     ('dotted',                (0, (1, 1))),
     ('densely dotted',        (0, (5, 1))),

     ('loosely dashed',        (0, (5, 10))),
     ('dashed',                (0, (5, 5))),
     ('densely dashed',        (0, (5, 1))),

     ('loosely dashdotted',    (0, (3, 10, 1, 10))),
     ('dashdotted',            (0, (3, 5, 1, 5))),
     ('densely dashdotted',    (0, (3, 1, 1, 1))),

     ('dashdotdotted',         (0, (3, 5, 1, 5, 1, 5))),
     ('loosely dashdotdotted', (0, (3, 10, 1, 10, 1, 10))),
     ('densely dashdotdotted', (0, (3, 1, 1, 1, 1, 1)))]
linestyle_length = int(len(list(enumerate(linestyle_tuple))))
y_axis_limits = [[1, 2],[-2, 2]]
H = 0.0127

# ============================ Velocity profiles ============================ #
x_lab = ['$x/H+2U/{{U}_{\mathrm{ref}}}$','$x/H+20V/{{U}_{\mathrm{ref}}}$']
fig_names = ['U','V']
locs = [-4, 1, 4, 6, 10]
minus_plus = ['m', '', '', '', '']
factor = [2, 20]

error_rel_max = np.zeros((len(x_lab),len(locs),len(cases)))

for i in range(len(x_lab)):
    
    fig, ax = plt.subplots(figsize=(10, 5))
    ax.set_aspect('equal', adjustable='box')
    # plot the backward facing step geometry
    ax.plot([-130,0,0,50], [1,1,0,0], '-', color = 'black', linewidth = 3)
    ax.plot([-130,50], [9,9], '-', color = 'black', linewidth = 3)
    ax.fill_between([-130, 0], -1, 1, facecolor='gray')

    for j in range(len(locs)):
        
        for k in range(len(cases)):
            
            address = 'saved/' + cases[k] + '/postProcessing/sample/2000/'
            file_name_num = address + 'x_by_h_' + minus_plus[j] + \
                        str(abs(locs[j])).zfill(2) + '_U.xy'
            num_data = np.loadtxt(file_name_num)

            u_ref = np.loadtxt(address + 'Uref_U.xy')
            u_ref = u_ref[1]

            ax.plot([locs[j],locs[j]], [int(locs[j]<0),9], ':', color = 'black',
                    linewidth = 0.5)
            ax.plot(locs[j] + factor[i]*num_data[:,i+1]/u_ref, num_data[:,0]/H,
                    linestyle = list(enumerate(linestyle_tuple))[k % linestyle_length][1][1],
                    label = cases_names[k])

            if k == 0:
                num_data_Ref = num_data[:,i+1]
            else:
                #error_rel_max[i,j,k] = np.mean(abs(num_data_Ref - num_data[:,i+1])/u_ref*100)
                #error_rel_max[i,j,k] = np.mean(abs((num_data_Ref - num_data[:,i+1]))/np.mean(abs(num_data_Ref)))*100
                #Mean sqare error (https://www.geeksforgeeks.org/python-mean-squared-error/):
                error_rel_max[i,j,k] = np.square(np.subtract(num_data_Ref,num_data[:,i+1])).mean()

        file_name_exp = 'Experiments/profiles_x_by_h_' + minus_plus[j] + \
                        str(abs(locs[j])).zfill(2) + '.dat'
        exp_data = np.loadtxt(file_name_exp,comments='#')
        ax.plot(locs[j] + factor[i]*exp_data[:,i+2], exp_data[:,1],
                marker='o', markerfacecolor='None', markeredgecolor='blue',
                markersize=5, linestyle = 'None', 
                label = 'Experiment')

    handles, labels = ax.get_legend_handles_labels()
    ax.set_xlim((-6, 12))
    ax.set_ylim((-0.02, 9.04))
    ax.set_xlabel(x_lab[i])
    ax.set_ylabel('$y/H$')
    display = list(range(len(cases)+1))
    ax.legend([handle for i,handle in enumerate(handles) if i in display],
              [label for i,label in enumerate(labels) if i in display],
               loc='best')
    plt.savefig(fig_names[i] + '.' + fig_save, format=fig_save)
    # plt.show()

# ================================= Cp & Cf ================================= #
rho = 1
num_name = ['p', 'wallShearStress']
exp_name = ['cp', 'cf']
wall_name = ['lower', 'upper']
plus_minus = [1, -1]
y_lab = ['$C_p$','$C_f$']

error_rel_max_wall = np.zeros((len(num_name),len(wall_name),len(cases)))

for i in range(len(num_name)):

    for j in range(len(wall_name)):
    
        fig, ax = plt.subplots(figsize=(6, 5))
        # ax.set_aspect('equal', adjustable='box')
        # plot the backward facing step geometry
        
        for k in range(len(cases)):
            
            address = 'saved/' + cases[k] + '/postProcessing/surface/2000/'
            file_name_num = address + num_name[i] + '_' + wall_name[j] + \
                        'Wall.raw'
            num_data = np.loadtxt(file_name_num)

            u_ref = np.loadtxt(address.replace('surface','sample') + 'Uref_U.xy')
            u_ref = u_ref[1]

            nondim_data = plus_minus[i]*num_data[:,3]/(0.5*rho*u_ref**2)
            ax.plot(num_data[:,0]/H, nondim_data,
                    linestyle = list(enumerate(linestyle_tuple))[k % linestyle_length][1][1],
                    label = cases_names[k])

            if k == 0:
                num_data_Ref = nondim_data
            else:
                #error_rel_max[i+2,j,k] = np.max(abs(num_data_Ref - nondim_data)/abs(np.mean(num_data_Ref)))*100
                #error_rel_max_wall[i,j,k] = np.mean(abs((num_data_Ref - nondim_data))/np.mean(abs(num_data_Ref)))*100
                #Mean sqare error (https://www.geeksforgeeks.org/python-mean-squared-error/):
                error_rel_max_wall[i,j,k] = np.square(np.subtract(num_data_Ref,nondim_data)).mean()

        if i + j < 2: # no experimental data for cf on upper wall
            file_name_exp = 'Experiments/' + exp_name[i] + '_' + \
                            wall_name[j] + '.dat'
            exp_data = np.loadtxt(file_name_exp,comments='#')
            ax.plot(exp_data[:,0], exp_data[:,1], marker='o',
                    markerfacecolor='None', markeredgecolor='blue',
                    markersize=5, linestyle = 'None', label = 'Experiment')

        ax.set_xlim((-5, 30))
        # ax.set_ylim((-0.02, 9.04))
        ax.set_xlabel('$x/H$')
        ax.set_ylabel(y_lab[i])
        ax.legend(loc='best')
        plt.savefig(exp_name[i] + '_' + wall_name[j] + '.' + fig_save, 
                    format=fig_save, bbox_inches='tight')

# =============================== Error report ============================== #

print('')
for k in range(len(cases)):
    if k == 0:
        print('Using case "' + cases[k] + '" as reference')
        print('Passing criteria: ' + str(criteria) + '%.')
        print('Relative errors:')
    else:
        error_max = max(np.max(error_rel_max[:,:,k]),np.max(error_rel_max_wall[:,:,k]))
        print(cases[k] + ': '
              "{:.2f}".format(error_max) + '% - ' +
              'PASS ' if error_max < criteria
                        else
              cases[k] + ': ' +
              "{:.2f}".format(error_max) + '% - ' +
              'FAIL.')
print('')

np.set_printoptions(precision=2)
np.set_printoptions(suppress=True)
print('Detailed report of FAILED cases:')
for k in range(len(cases)):
    error_max = max(np.max(error_rel_max[:,:,k]),np.max(error_rel_max_wall[:,:,k]))
    if error_max >= criteria:
        print('==================')
        print(cases[k] + ': ')
        print('Rows:', fig_names, 'Columns: Locations',locs)
        print(error_rel_max[:,:,k])
        print('Rows:', exp_name, 'Columns: Locations',wall_name)
        print(error_rel_max_wall[:,:,k])
print('==================')
